/*
 * maevies_window.c
 *
 * This file is part of maevies
 * Copyright (C) 2009 spenap <spenap@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 */

#include "maevies_window.h"

static void hello_item_clicked(GtkButton *button, gpointer data);

G_DEFINE_TYPE(MaeviesWindow, maevies_window, HILDON_TYPE_WINDOW)

static void maevies_window_dispose(GObject *object) {

	MaeviesWindow *self = MAEVIES_WINDOW(object);

	/* Free member data.
	 * Note that the child widgets are destroyed automatically.
	 */

	G_OBJECT_CLASS (maevies_window_parent_class)->dispose(object);
}

static void maevies_window_finalize(GObject *object) {
	G_OBJECT_CLASS (maevies_window_parent_class)->finalize(object);
}

static void maevies_window_class_init(MaeviesWindowClass *klass) {
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->dispose = maevies_window_dispose;
	object_class->finalize = maevies_window_finalize;
}

static void maevies_window_init(MaeviesWindow *self) {

	/* Create button and add it to main view */
	self->hello_item = gtk_button_new_with_label("Hello World!!!");
	gtk_container_add(GTK_CONTAINER(self), self->hello_item);

	g_signal_connect(G_OBJECT(self->hello_item), "clicked", G_CALLBACK(hello_item_clicked),
			self);

	/* Init movie */
	self->movie = maevies_movie_new("Zombieland");
}

MaeviesWindow* maevies_window_new(osso_context_t *osso) {
	MaeviesWindow *self = MAEVIES_WINDOW(g_object_new(MAEVIES_TYPE_WINDOW,
					NULL));

	/* Avoid adding extra code such as this to a _new() function when writing
	 * widgets that should be reusable. This should really be a GObject property.
	 */
	self->osso = osso;

	return self;
}

static void hello_item_clicked(GtkButton* button, gpointer data) {

	MaeviesWindow *self = MAEVIES_WINDOW(data);
	g_assert(self);

	movie_get_info(self->movie, NULL);

	gchar *has_stingers = NULL;

	if (self->movie->has_stingers)
		has_stingers = "Con escenas";
	else
		has_stingers = "Sin escenas";

	gtk_button_set_label(button, has_stingers);
}
